import typer
import httpx
import lxml.html
from typing_extensions import Annotated
from .db import initialize_db
from .tui.things import run as things_tui
#from .tui.overview import run as overview_tui
from .tui.recurring import run as recurring_tui
from .controller.things import add_thing

app = typer.Typer()


@app.command()
def new(
    name: Annotated[str, typer.Option("-n", "--name", help="name")] = "",
    category: Annotated[str, typer.Option("-c", "--category", help="name")] = "",
    due: Annotated[str, typer.Option("-d", "--due", help="due")] = "",
    url: Annotated[str, typer.Option("-u", "--url", help="URL")] = "",
):
    initialize_db()
    required = ["name", "category"]

    if url and not name:
        resp = httpx.get(url, follow_redirects=True)
        doc = lxml.html.fromstring(resp.text)
        try:
            name = doc.xpath("//title/text()")[0]
            print(f"fetched {url} name... {name}")
        except IndexError:
            name = ""

    if "name" in required and not name:
        name = typer.prompt("Name")
    if "category" in required and not category:
        category = typer.prompt("Category")
    # due = typer.prompt("Due (YYYY-MM-DD):")
    # TODO: validate/allow blank

    add_thing(name, category, due)
    typer.echo("Created new thing!")

@app.command()
def table(
    view: Annotated[str, typer.Option("-v", "--view", help="saved view")] = "default",
):
    initialize_db()
    things_tui("tasks")


@app.command()
def generators():
    initialize_db()
    recurring_tui()


@app.command()
def overview():
    initialize_db()
    overview_tui()


if __name__ == "__main__":
    app()
