from textual.screen import ModalScreen
from rich.table import Table
from textual.widgets import Static


class KeyModal(ModalScreen):
    CSS = """
    KeyBindingsScreen {
        align: center middle;
    }

    Vertical {
        height: auto;
        border: tall $primary;
    }

    Static.title {
        text-align: center;
        text-style: bold;
    }

    Static.footer {
        text-align: center;
        color: $text-muted;
    }
    """

    def compose(self):
        table = Table(expand=True, show_header=True)
        table.add_column("Key")
        table.add_column("Description")

        table.add_row("j/k/h/l", "up/down/left/right")
        table.add_row("g/G", "top/bottom")
        table.add_row("esc", "dismiss modal or search bar")

        for binding in self.app.BINDINGS:
            if binding[0] not in ["h", "j", "k", "l", "g", "G", "escape"]:
                table.add_row(binding[0], binding[2])
        # TODO: MRO?

        yield Static("tt keybindings", classes="title")
        yield Static(table)
        yield Static("Press any key to dismiss", classes="footer")

    def on_key(self, event) -> None:
        self.app.pop_screen()
